@@ -104,7 +104,6 @@ void not_found_msg(const char *str)
  * search, if any. */
 void search_replace_abort(void)
 {
-    display_main_list();
 #ifndef NANO_TINY
     if (openfile->mark_set)
 	edit_refresh();
@@ -302,6 +301,8 @@ int findnextstr(
 	    }
 
 	    if (++feedback > 0)
+		/* TRANSLATORS: This is shown when searching takes
+		 * more than half a second. */
 		statusbar(_("Searching..."));
 	}
 
@@ -628,7 +629,7 @@ ssize_t do_replace_loop(
     if (old_mark_set) {
 	/* If the mark is on, frame the region, and turn the mark off. */
 	mark_order((const filestruct **)&top, &top_x,
-		(const filestruct **)&bot, &bot_x, &right_side_up);
+			(const filestruct **)&bot, &bot_x, &right_side_up);
 	openfile->mark_set = FALSE;
 
 	/* Start either at the top or the bottom of the marked region. */
@@ -913,31 +914,27 @@ void do_gotolinecolumn(ssize_t line, ssize_t column, bool use_answer,
 		/* TRANSLATORS: This is a prompt. */
 		edit_refresh, _("Enter line number, column number"));
 
-	/* Cancel, or Enter with blank string. */
+	/* If the user cancelled or gave a blank answer, get out. */
 	if (i < 0) {
 	    statusbar(_("Cancelled"));
-	    display_main_list();
 	    return;
 	}
 
 	func = func_from_key(&i);
 
 	if (func == gototext_void) {
-	    /* Keep answer up on the statusbar. */
+	    /* Retain what the user typed so far and switch to searching. */
 	    search_init(TRUE, TRUE);
-
 	    do_search();
-	    return;
 	}
 
-	/* Do a bounds check.  Display a warning on an out-of-bounds
-	 * line or column number only if we hit Enter at the statusbar
-	 * prompt. */
-	if (!parse_line_column(answer, &line, &column) ||
-			line < 1 || column < 1) {
-	    if (i == 0)
-		statusbar(_("Invalid line or column number"));
-	    display_main_list();
+	/* If a function was executed, we're done here. */
+	if (i > 0)
+	    return;
+
+	/* Try to extract one or two numbers from the user's response. */
+	if (!parse_line_column(answer, &line, &column)) {
+	    statusbar(_("Invalid line or column number"));
 	    return;
 	}
     } else {
@@ -948,10 +945,24 @@ void do_gotolinecolumn(ssize_t line, ssize_t column, bool use_answer,
 	    column = openfile->placewewant + 1;
     }
 
+    /* Take a negative line number to mean: from the end of the file. */
+    if (line < 0)
+	line = openfile->filebot->lineno + line + 1;
+    if (line < 1)
+	line = 1;
+
+    /* Iterate to the requested line. */
     for (openfile->current = openfile->fileage; line > 1 &&
 		openfile->current != openfile->filebot; line--)
 	openfile->current = openfile->current->next;
 
+    /* Take a negative column number to mean: from the end of the line. */
+    if (column < 0)
+	column = strlenpt(openfile->current->data) + column + 2;
+    if (column < 1)
+	column = 1;
+
+    /* Set the x position that corresponds to the requested column. */
     openfile->current_x = actual_x(openfile->current->data, column - 1);
     openfile->placewewant = column - 1;
 
@@ -959,7 +970,6 @@ void do_gotolinecolumn(ssize_t line, ssize_t column, bool use_answer,
     if (interactive) {
 	edit_update(CENTERING);
 	edit_refresh();
-	display_main_list();
     } else {
 	/* If the target line is close to the tail of the file, put the last
 	 * line of the file on the bottom line of the screen; otherwise, just
